package flink.zanxiangnet.ad.monitoring.process;

import flink.zanxiangnet.ad.monitoring.pojo.entity.AdStatOfHourDWD;
import flink.zanxiangnet.ad.monitoring.pojo.entity.CostHourDM;
import flink.zanxiangnet.ad.monitoring.util.DateUtil;
import org.apache.flink.api.common.state.MapState;
import org.apache.flink.api.common.state.MapStateDescriptor;
import org.apache.flink.api.common.typeinfo.Types;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.KeyedProcessFunction;
import org.apache.flink.util.Collector;

import java.io.IOException;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.*;

public class CostHourProcess extends KeyedProcessFunction<Long, AdStatOfHourDWD, CostHourDM> {

    private MapState<String, Map<Integer, AdStatOfHourDWD>> historyReduceState;


    @Override
    public void open(Configuration conf) throws SQLException, ClassNotFoundException, IOException {
        historyReduceState = getRuntimeContext().getMapState(new MapStateDescriptor<>("historyReduceState", Types.STRING, Types.MAP(Types.INT, Types.POJO(AdStatOfHourDWD.class))));
    }

    @Override
    public void processElement(AdStatOfHourDWD adStatOfHourDWD, KeyedProcessFunction<Long, AdStatOfHourDWD, CostHourDM>.Context context,
                               Collector<CostHourDM> collector) throws Exception {
        LocalDate day = DateUtil.parseLocalDate(adStatOfHourDWD.getStatDay());
        Integer hour = adStatOfHourDWD.getHour();

        LocalDate lastHourDay = day, lastTwoHourDay = day, lastThreeHourDay = day;
        int lastHour = hour - 1, lastTwoHour = hour - 2, lastThreeHour = hour - 3;
        if (hour == 0) {
            lastHourDay = day.minusDays(1L);
            lastHour = 23;
            lastTwoHourDay = day.minusDays(1L);
            lastTwoHour = 22;
            lastThreeHourDay = day.minusDays(1L);
            lastThreeHour = 21;
        } else if (hour == 1) {
            lastTwoHourDay = day.minusDays(1L);
            lastTwoHour = 23;
            lastThreeHourDay = day.minusDays(1L);
            lastThreeHour = 22;
        } else if (hour == 2) {
            lastThreeHourDay = day.minusDays(1L);
            lastThreeHour = 23;
        }
        long costDiff = 0L, costLastHour = 0L;
        Map<Integer, AdStatOfHourDWD> lastHourMapping = historyReduceState.get(DateUtil.formatLocalDate(lastHourDay));
        if (lastHourMapping != null && !lastHourMapping.isEmpty()) {
            AdStatOfHourDWD lastHourDWD = lastHourMapping.get(lastHour);
            if (lastHourDWD != null) {
                costLastHour = lastHourDWD.getCostHour();
                costDiff = adStatOfHourDWD.getCostHour() - lastHourDWD.getCostHour();
            }
        }
        long costLastHourDiff = 0, costLastTwoHour = 0;
        Map<Integer, AdStatOfHourDWD> lastTwoHourMapping = historyReduceState.get(DateUtil.formatLocalDate(lastTwoHourDay));
        if (lastTwoHourMapping != null && !lastTwoHourMapping.isEmpty()) {
            AdStatOfHourDWD lastTwoHourDWD = lastTwoHourMapping.get(lastTwoHour);
            if (lastTwoHourDWD != null) {
                costLastTwoHour = lastTwoHourDWD.getCostHour();
                costLastHourDiff = costLastHour - lastTwoHourDWD.getCostHour();
            }
        }
        long costLastTwoHourDiff = 0, costLastThreeTrend = 0, costLastThreeHour = 0;
        Map<Integer, AdStatOfHourDWD> lastThreeHourMapping = historyReduceState.get(DateUtil.formatLocalDate(lastThreeHourDay));
        if (lastThreeHourMapping != null && !lastThreeHourMapping.isEmpty()) {
            AdStatOfHourDWD lastThreeHourDWD = lastThreeHourMapping.get(lastThreeHour);
            if (lastThreeHourDWD != null) {
                costLastThreeTrend = costLastHourDiff > 0 && costDiff > 0 ? 1 : 0;
                costLastThreeHour = lastThreeHourDWD.getCostHour();
                costLastTwoHourDiff = costLastTwoHour - lastThreeHourDWD.getCostHour();
            }
        }

        if (historyReduceState.get(adStatOfHourDWD.getStatDay()) == null) {
            Map<Integer, AdStatOfHourDWD> hourMapping = new HashMap<>(24);
            hourMapping.put(adStatOfHourDWD.getHour(), adStatOfHourDWD);
            historyReduceState.put(adStatOfHourDWD.getStatDay(), hourMapping);
        } else {
            historyReduceState.get(adStatOfHourDWD.getStatDay()).put(adStatOfHourDWD.getHour(), adStatOfHourDWD);
        }

        CostHourDM costHourDM = dataChange(adStatOfHourDWD);
        costHourDM.setCostDiff(costDiff);
        costHourDM.setCostLastHour(costLastHour);
        costHourDM.setCostLastHourDiff(costLastHourDiff);
        costHourDM.setCostLastTwoHour(costLastTwoHour);
        costHourDM.setCostLastTwoHourDiff(costLastTwoHourDiff);
        costHourDM.setCostLastThreeTrend(costLastThreeTrend);
        costHourDM.setCostLastThreeHour(costLastThreeHour);


        collector.collect(costHourDM);
        historyReduceState.remove(DateUtil.formatLocalDate(day.minusDays(2L)));
        historyReduceState.remove(DateUtil.formatLocalDate(day.minusDays(3L)));

    }

    //数据格式转换
    public CostHourDM dataChange(AdStatOfHourDWD adStatOfMinuteDWD) {
        CostHourDM costHourDM = new CostHourDM();

        //时间-天
        costHourDM.setDt(adStatOfMinuteDWD.getStatDay());
        //计划 id
        costHourDM.setCampaignId(adStatOfMinuteDWD.getCampaignId().toString());
        //时间- real
        costHourDM.setCreateTime(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(adStatOfMinuteDWD.getCreateTime()));
        //时间-小时
        //TODO:之后需要进一步修改
        String tmpHour = adStatOfMinuteDWD.getHour() > 9 ? adStatOfMinuteDWD.getHour().toString() : "0" + adStatOfMinuteDWD.getHour().toString();
        costHourDM.setHour(adStatOfMinuteDWD.getStatDay() + " " + tmpHour + ":00:00");
        //广告id
        costHourDM.setAdId(adStatOfMinuteDWD.getAdId().toString());
        //服务商id
        costHourDM.setAgencyAccountId(adStatOfMinuteDWD.getAgencyAccountId().toString());
        //广告组id
        costHourDM.setAdgroupId(adStatOfMinuteDWD.getAdgroupId().toString());
        //创意id
        costHourDM.setAdcreativeId("");
        //账号id
        costHourDM.setAccountId(adStatOfMinuteDWD.getAccountId().toString());
        //总消耗
        costHourDM.setCostTotal(adStatOfMinuteDWD.getCostTotal());
        //当天消耗
        costHourDM.setCostDay(adStatOfMinuteDWD.getCostDay());
        //当天小时消耗
        costHourDM.setCostHour(adStatOfMinuteDWD.getCostHour());
        //消耗速度
        costHourDM.setCostSpeed(adStatOfMinuteDWD.getCostHour());
        //总浏览量
        costHourDM.setViewCountTotal(adStatOfMinuteDWD.getViewCountTotal());
        //天-总浏览量
        costHourDM.setViewCountDay(adStatOfMinuteDWD.getViewCountDay());
        //小时-总浏览量
        costHourDM.setViewCountHour(adStatOfMinuteDWD.getViewCountHour());
        //总平均千次曝光成本
        costHourDM.setThousandDisplayPriceAll(adStatOfMinuteDWD.getThousandDisplayPriceAll());
        //天-总平均曝光成本
        costHourDM.setThousandDisplayPriceDay(adStatOfMinuteDWD.getThousandDisplayPriceDay());
        //小时-总平均曝光成本
        costHourDM.setThousandDisplayPriceHour(adStatOfMinuteDWD.getThousandDisplayPriceHour());
        //总点击量
        costHourDM.setValidClickCountTotal(adStatOfMinuteDWD.getValidClickCountTotal());
        //天-总点击量
        costHourDM.setValidClickCountDay(adStatOfMinuteDWD.getValidClickCountDay());
        //小时-总点击量
        costHourDM.setValidClickCountHour(adStatOfMinuteDWD.getValidClickCountHour());
        //总平均点击率
        costHourDM.setCtrAll(adStatOfMinuteDWD.getCtrAll());
        //天-总平均点击率
        costHourDM.setCtrDay(adStatOfMinuteDWD.getCtrDay());
        //小时-总平均点击率
        costHourDM.setCtrHour(adStatOfMinuteDWD.getCtrHour());
        //总点击均价
        costHourDM.setCpcAll(adStatOfMinuteDWD.getCpcAll());
        //天-总点击均价
        costHourDM.setCpcDay(adStatOfMinuteDWD.getCpcDay());
        //小时-总点击均价
        costHourDM.setCpcHour(adStatOfMinuteDWD.getCpcHour());
        //总目标转化量
        costHourDM.setConversionsCountTotal(adStatOfMinuteDWD.getConversionsCountTotal());
        //天-总目标转化量
        costHourDM.setConversionsCountDay(adStatOfMinuteDWD.getConversionsCountDay());
        //小时-总目标转化量
        costHourDM.setConversionsCountHour(adStatOfMinuteDWD.getConversionsCountHour());
        //总目标平均转化成本
        costHourDM.setConversionsCostTotal(adStatOfMinuteDWD.getConversionsCostAll());
        //天-总目标平均转化成本
        costHourDM.setConversionsCostDay(adStatOfMinuteDWD.getConversionsCostDay());
        //小时-总目标平均转化成本
        costHourDM.setConversionsCostHour(adStatOfMinuteDWD.getConversionsCostHour());
        //总平均转化率
        costHourDM.setConversionsRateAll(adStatOfMinuteDWD.getConversionsRateAll());
        //天-总平均转化率
        costHourDM.setConversionsRateDay(adStatOfMinuteDWD.getConversionsRateDay());
        //小时-总平均转化率
        costHourDM.setConversionsRateHour(adStatOfMinuteDWD.getConversionsRateHour());
        //TODO:总首日下单roi
        costHourDM.setFirstDayOrderRoiTotal(0);
        //天-总首日下单roi
        costHourDM.setFirstDayOrderRoiDay(0);
        //小时-总首日下单roi
        costHourDM.setFirstDayOrderRoiHour(0);
        //总首日下单金额
        costHourDM.setFirstDayOrderAmountTotal(adStatOfMinuteDWD.getFirstDayOrderAmountTotal());
        //天-总首日下单金额
        costHourDM.setFirstDayOrderAmountDay(adStatOfMinuteDWD.getFirstDayOrderAmountDay());
        //小时-总首日下单金额
        costHourDM.setFirstDayOrderAmountHour(adStatOfMinuteDWD.getFirstDayOrderAmountHour());
        //总首日下单量
        costHourDM.setFirstDayOrderCountTotal(adStatOfMinuteDWD.getFirstDayOrderCountTotal());
        //天-总首日下单量
        costHourDM.setFirstDayOrderCountDay(adStatOfMinuteDWD.getFirstDayOrderCountDay());
        //小时-总首日下单量
        costHourDM.setFirstDayOrderCountHour(adStatOfMinuteDWD.getFirstDayOrderCountHour());
        //总下单金额
        costHourDM.setWebOrderAmountTotal(adStatOfMinuteDWD.getOrderAmountTotal());
        //天-总下单金额
        costHourDM.setWebOrderAmountDay(adStatOfMinuteDWD.getOrderAmountDay());
        //小时-总下单金额
        costHourDM.setWebOrderAmountHour(adStatOfMinuteDWD.getOrderAmountHour());
        //总平均下单成本
        costHourDM.setWebOrderCostTotal(adStatOfMinuteDWD.getWebOrderCostAll());
        //天-总平均下单成本
        costHourDM.setWebOrderCostDay(adStatOfMinuteDWD.getWebOrderCostDay());
        //小时-总平均下单成本
        costHourDM.setWebOrderCostHour(adStatOfMinuteDWD.getWebOrderCostHour());
        //总平均下单率
        costHourDM.setWebOrderRateTotal(adStatOfMinuteDWD.getOrderRateAll());
        //天-总平均下单率
        costHourDM.setWebOrderRateDay(adStatOfMinuteDWD.getOrderRateDay());
        //小时-总平均下单率
        costHourDM.setWebOrderRateHour(adStatOfMinuteDWD.getOrderRateHour());
        //TODO:总平均下单量-----webordercount和ordercount是同一个东西吗
        costHourDM.setWebOrderCountTotal(adStatOfMinuteDWD.getOrderCountTotal());
        //天-总平均下单量
        costHourDM.setWebOrderCountDay(adStatOfMinuteDWD.getOrderCountDay());
        //小时-总平均下单量
        costHourDM.setWebOrderCountHour(adStatOfMinuteDWD.getOrderCountHour());
        //总下单ROI
        costHourDM.setOrderRoiTotal(adStatOfMinuteDWD.getOrderRoiAll());
        //天-总下单roi
        costHourDM.setOrderRoiDay(adStatOfMinuteDWD.getOrderRoiDay());
        //小时-总下单roi
        costHourDM.setOrderRoiHour(adStatOfMinuteDWD.getOrderRoiHour());
        //总平均下单客单价
        costHourDM.setOrderUnitPriceTotal(adStatOfMinuteDWD.getOrderUnitPriceAll());
        //天-总平均下单客单价
        costHourDM.setOrderUnitPriceDay(adStatOfMinuteDWD.getOrderUnitPriceDay());
        //小时-总平均下单客单价
        costHourDM.setOrderUnitPriceHour(adStatOfMinuteDWD.getOrderUnitPriceHour());
        //总公众号关注量
        costHourDM.setFromFollowUvTotal(adStatOfMinuteDWD.getOfficialAccountFollowCountTotal());
        //天-总公众号关注量
        costHourDM.setFromFollowUvDay(adStatOfMinuteDWD.getOfficialAccountFollowCountDay());
        //小时-总公众号关注量
        costHourDM.setFromFollowUvHour(adStatOfMinuteDWD.getOfficialAccountFollowCountHour());
        //TODO:总平均公众号关注成本---是否是价格/关注
        costHourDM.setFromFollowCostTotal(adStatOfMinuteDWD.getOfficialAccountFollowCountTotal() == 0 ? 0 : adStatOfMinuteDWD.getCostTotal() / adStatOfMinuteDWD.getOfficialAccountFollowCountTotal());
        //天-总平均公众号关注成本
        costHourDM.setFromFollowCostDay(adStatOfMinuteDWD.getOfficialAccountFollowCountDay() == 0 ? 0 : adStatOfMinuteDWD.getCostDay() / adStatOfMinuteDWD.getOfficialAccountFollowCountDay());
        //小时-总平均公众号关注成本
        costHourDM.setFromFollowCostHour(adStatOfMinuteDWD.getOfficialAccountFollowCountHour() == 0 ? 0 : adStatOfMinuteDWD.getCostHour() / adStatOfMinuteDWD.getOfficialAccountFollowCountHour());
        //TODO:总平均公众号关注率----确认是否对应
        costHourDM.setFromFollowRateTotal(adStatOfMinuteDWD.getOfficialAccountFollowRateAll());
        //天-总平均公众号关注率
        costHourDM.setFromFollowRateDay(adStatOfMinuteDWD.getOfficialAccountFollowRateDay());
        //小时-总平均公众号关注率
        costHourDM.setFromFollowRateHour(adStatOfMinuteDWD.getOfficialAccountFollowRateHour());
        //TODO:总注册数-----下面全是有问题的
        costHourDM.setWebRegisterCountTotal(adStatOfMinuteDWD.getOfficialAccountRegisterUserCountTotal());
        //天-总注册数
        costHourDM.setWebRegisterCountDay(adStatOfMinuteDWD.getOfficialAccountRegisterUserCountDay());
        //小时-总注册数
        costHourDM.setWebRegisterCountHour(adStatOfMinuteDWD.getOfficialAccountRegisterUserCountHour());
        //总注册人数
        costHourDM.setWebRegisterUvTotal(adStatOfMinuteDWD.getOfficialAccountRegisterUserCountTotal());
        //天-总注册人数
        costHourDM.setWebRegisterUvDay(adStatOfMinuteDWD.getOfficialAccountRegisterUserCountDay());
        //小时-总注册人数
        costHourDM.setWebRegisterUvHour(adStatOfMinuteDWD.getOfficialAccountRegisterUserCountHour());
        //总平均注册成本
        costHourDM.setWebRegisterCostTotal(adStatOfMinuteDWD.getOfficialAccountRegisterCostAll());
        //天-总平均注册成本
        costHourDM.setWebRegisterCostDay(adStatOfMinuteDWD.getOfficialAccountRegisterCostDay());
        //小时-总平均注册成本
        costHourDM.setWebRegisterCostHour(adStatOfMinuteDWD.getOfficialAccountRegisterCostHour());
        return costHourDM;
    }
}
